/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* Jprobe.java,v 1.5 2003/10/13 21:54:26 jba Exp */
/**
 * This is the Jprobe application.
 *
 * @author <A HREF="mailto:jca@aps.anl.gov">
 * Janet Anderson</A>
 *
*/
package jprobe;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.border.*;

import java.awt.*;
import java.awt.event.*;

import jca.*;

public class Jprobe extends JFrame implements ValueEventListener
{
  protected JprobeAdjustDialog  adjustDialog = null;
  protected JprobeFormatDialog  formatDialog = null;
  protected JprobeInfoDialog    infoDialog = null;
  protected JprobePv            jprobePv;
  protected JLabel              pvNameLabel;
  protected JTextField          pvNameTextField;
  protected JLabel              pvStateLabel;
  protected JLabel              pvValueLabel;
  protected JButton             startStopButton = null;
  protected JprobeVersionDialog versionDialog = null;

  /** Constructor with one argument
  * It creates the Jprobe Dialog
  * @param text is the pv name.
  */
  public Jprobe(String text)
  {
    super("Jprobe");

    //Font pvNameFont = new Font("Courier",Font.BOLD,16);
    //Font pvValueFont = new Font("TimesRoman",Font.BOLD,24);
    //Font pvStateFont = new Font("TimesRoman",Font.BOLD,16);

    this.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e)
      {
      jprobePv.removeAllValueEventListeners();
      try { 
        jprobePv.clear();
      }
      catch (jca.BadMonitorException bm){popup("jca.BadMonitorException"); }
      catch (jca.TimeOutException to){popup("jca.TimeOutException"); }
      Ca.exit();
      System.exit(0);}
    });

    JPanel entryPanel = new JPanel();
    entryPanel.setLayout(new GridLayout(0,1));
    entryPanel.setBorder(new EmptyBorder(10,10,10,10));

    /* add labels for pv name, value,and status */
    pvNameLabel = new JLabel("Channel name");
    //pvNameLabel.setFont(pvNameFont);
    entryPanel.add(pvNameLabel);
    pvValueLabel = new JLabel("Channel value");
    pvValueLabel.setFont(new Font(pvValueLabel.getFont().getName(),Font.BOLD,24));
    //pvValueLabel.setFont(pvValueFont);
    entryPanel.add(pvValueLabel);
    pvStateLabel = new JLabel("status");
    //pvStateLabel.setFont(pvStateFont);
    entryPanel.add(pvStateLabel);

    try { 
      this.jprobePv  = new JprobePv(text,this);
    }
    catch (jca.BadCountException bc){popup("jca.BadCountException"); }
    catch (jca.BadTypeException bt){popup("jca.BadTypeException"); }
    catch (jca.BadMonitorException bm){popup("jca.BadMonitorException"); }
    catch (jca.GetFailException gf){popup("jca.GetFailException"); }
    catch (jca.MonitorFailException bm){popup("jca.MonitorFailException"); }
    catch (jca.PutFailException gf){popup("jca.PutFailException"); }
    catch (jca.TimeOutException to){popup("jca.TimeOutException"); }



    /* add a pv name text entry field */
    pvNameTextField = new JTextField(text,30);
    entryPanel.add(pvNameTextField);
    pvNameTextField.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        String pvName = ((JTextField)e.getSource()).getText();
        try { 
          jprobePv.setName(pvName);
        }
        catch (jca.BadCountException bc){popup("jca.BadCountException"); }
        catch (jca.BadMonitorException bm){popup("jca.BadMonitorException"); }
        catch (jca.BadTypeException bt){popup("jca.BadTypeException"); }
        catch (jca.GetFailException gf){popup("jca.GetFailException"); }
        catch (jca.MonitorFailException bm){popup("jca.MonitorFailException"); }
        catch (jca.PutFailException gf){popup("jca.PutFailException"); }
        catch (jca.TimeOutException to){popup("jca.TimeOutException"); }
        if (startStopButton != null) startStopButton.setText("Start");
      }
    });

    this.getContentPane().setLayout(new BorderLayout());
    this.getContentPane().add(entryPanel, BorderLayout.CENTER);

    Border emptyBorder = new EmptyBorder(0,5,0,5);
    Border raisedBorder = new BevelBorder(BevelBorder.RAISED);

    /* add a Start button */
    JPanel panel = new JPanel(new GridLayout(2,3,6,6));
    panel.setBorder(new CompoundBorder(new EtchedBorder(),
                    new EmptyBorder(5,5,5,5)));
    JButton button;
    button = new JButton("Start");
    startStopButton = button;
    button.setBorder(new CompoundBorder(raisedBorder,emptyBorder));
    button.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        JButton b=(JButton)e.getSource();
        String s=b.getText();
        if (s=="Start") {
          try { 
            jprobePv.setMonitored(true);
          }
          catch (jca.BadCountException bc){popup("jca.BadCountException"); }
          catch (jca.BadMonitorException bm){popup("jca.BadMonitorException"); }
          catch (jca.BadTypeException bt){popup("jca.BadTypeException"); }
          catch (jca.GetFailException bt){popup("jca.GetFailException"); }
          catch (jca.MonitorFailException mf){popup("jca.MonitorFailException"); }
          catch (jca.TimeOutException to){popup("jca.TimeOutException"); }
	  b.setText("Stop");
        }else if (s=="Stop"){
          try { 
            jprobePv.setMonitored(false);
          }
          catch (jca.BadCountException bc){popup("jca.BadCountException"); }
          catch (jca.BadMonitorException bm){popup("jca.BadMonitorException"); }
          catch (jca.BadTypeException bt){popup("jca.BadTypeException"); }
          catch (jca.GetFailException bt){popup("jca.GetFailException"); }
          catch (jca.MonitorFailException mf){popup("jca.MonitorFailException"); }
          catch (jca.TimeOutException to){popup("jca.TimeOutException"); }
	  b.setText("Start");
        }
      }
    });
    panel.add(button);

    /* add an Adjust button */
    button = new JButton("Adjust");
    button.setBorder(new CompoundBorder(raisedBorder,emptyBorder));
    button.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        if (adjustDialog == null) {
          adjustDialog = new JprobeAdjustDialog(Jprobe.this,jprobePv);
          adjustDialog.setLocation(50,300);
          adjustDialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
              jprobePv.removeValueEventListener(adjustDialog);
              adjustDialog.dispose();
              adjustDialog = null;
            }
            public void windowClosed(WindowEvent e) { 
              jprobePv.removeValueEventListener(adjustDialog);
              adjustDialog = null; }
            });
        } else {
          adjustDialog.toFront();
        }
      }
    });
    panel.add(button);

    /* add an Info button */
    button = new JButton("Info");
    button.setBorder(new CompoundBorder(raisedBorder,emptyBorder));
    button.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        if (infoDialog == null) {
          infoDialog = new JprobeInfoDialog(Jprobe.this,jprobePv);
          infoDialog.setLocation(500,300);
          infoDialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
              jprobePv.removeValueEventListener(infoDialog);
              infoDialog.dispose();
              infoDialog = null;
            }
            public void windowClosed(WindowEvent e) { 
              jprobePv.removeValueEventListener(infoDialog);
              infoDialog = null; }
            });
        } else {
          infoDialog.toFront();
        }
      }
    });
    panel.add(button);

    /* add a Format button */
    button = new JButton("Format");
    button.setBorder(new CompoundBorder(raisedBorder,emptyBorder));
    button.addActionListener(new ActionListener() 
    {
      public void actionPerformed(ActionEvent e)
      {
        if (formatDialog == null) {
          formatDialog = new JprobeFormatDialog(Jprobe.this,jprobePv);
          formatDialog.setLocation(300,300);
          formatDialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
              jprobePv.removeValueEventListener(formatDialog);
              formatDialog.dispose();
              formatDialog = null;
            }
            public void windowClosed(WindowEvent e) { 
              jprobePv.removeValueEventListener(formatDialog);
              formatDialog = null; }
            });
        } else {
          formatDialog.toFront();
        }
      }
    });
    panel.add(button);

    /* add a Version button */
    button = new JButton("Version");
    button.setBorder(new CompoundBorder(raisedBorder,emptyBorder));
    button.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e) 
      {
        if (versionDialog == null) {
          versionDialog = new JprobeVersionDialog(Jprobe.this);
          versionDialog.setLocation(100,20);
            versionDialog.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
              versionDialog.dispose();
              versionDialog = null;
            }
            public void windowClosed(WindowEvent e) { 
              versionDialog = null; }
            });
        } else {
          versionDialog.toFront();
        }
      }
    });

    panel.add(button);

    /* add a Quit button */
    button = new JButton("Quit");
    button.setBorder(new CompoundBorder(raisedBorder,emptyBorder));
    button.addActionListener(new ActionListener()
    {
      public void actionPerformed(ActionEvent e)
      {
        jprobePv.removeAllValueEventListeners();
        try { 
          jprobePv.clear();
        }
        catch (jca.BadMonitorException bm){popup("jca.BadMonitorException"); }
        catch (jca.TimeOutException to){popup("jca.TimeOutException"); }
        Ca.exit();
        System.exit(0);
      }
    });
    panel.add(button);

    this.getContentPane().add(panel, BorderLayout.SOUTH);

    this.setLocation(100,100);
    //this.pack();
    this.setSize(300,200);
    this.setVisible(true);

/*
    try { 
      this.jprobePv  = new JprobePv(text,this);
    }
    catch (jca.BadCountException bc){popup("jca.BadCountException"); }
    catch (jca.BadTypeException bt){popup("jca.BadTypeException"); }
    catch (jca.BadMonitorException bm){popup("jca.BadMonitorException"); }
    catch (jca.GetFailException gf){popup("jca.GetFailException"); }
    catch (jca.MonitorFailException bm){popup("jca.MonitorFailException"); }
    catch (jca.PutFailException gf){popup("jca.PutFailException"); }
    catch (jca.TimeOutException to){popup("jca.TimeOutException"); }
*/

  }

  protected void popup(String text)
  {
     JOptionPane.showMessageDialog(this,text,"OK",JOptionPane.ERROR_MESSAGE);
  }

  /** valueChanged method
  * @param  e is the ValueEvent input argument
  */
  public void valueChanged(ValueEvent e)
  {
//    synchronized(this) {
      jprobePv = e.jprobePv();
//    }
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        updateDialog();
      }
    });
  }


  protected void updateDialog()
  {
     if (jprobePv.getName().equals("") ) {
       pvNameLabel.setText("Channel name");
       pvValueLabel.setText("Channel value");
       pvStateLabel.setText("status");
     }
     else {
       pvNameLabel.setText(jprobePv.getName());
       pvValueLabel.setText(jprobePv.getValueString()+" "+
                            jprobePv.getUnitsString());
       String monitorString = jprobePv.isMonitored()?"  monitored":"";
       pvStateLabel.setText(jprobePv.getAlarmStateString()+monitorString);
     }
  }

  /** main method
  * @param args are the command line arguments
  * the first command line argument will be the pv name
  */
  public static void main(String[] args)
  {
    //String vers = System.getProperty("java.version");
    //if (vers.compareTo("1.1.2") < 0) {
      //System.out.println("!!!WARNING: Jprobe must be run with a " +
      //                   "1.1.2 or higher version VM!!!");
    //}

    try {
       UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (Exception exc) {
      System.out.println("Error loading L&F: " + exc);
    }

    String pvname = "";
    if (args.length > 0){
      pvname = args[0];
    }
	Ca.init();
    Jprobe jprobe = new Jprobe(pvname);
    JprobePv.startCaLoop();
  }
}

