/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* JprobeAdjustDialog.java,v 1.5 2002/08/02 15:39:23 jba Exp */
/**
 * This is the Adjust dialog.
 *
 * @author <A HREF="mailto:jba@aps.anl.gov">
 * Janet Anderson</A>
 *
 * @version 1.0
*/

package jprobe;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import jca.*;
import jca.dbr.*;

public class JprobeAdjustDialog extends JDialog implements ValueEventListener
{
  protected static final int VALUE_TEXT_FIELD_LENGTH = 30; /* chars */

  protected JPanel     adjustPanel =  null;
  protected boolean    editing = false;
  protected JprobePv   jprobePv;
  protected String     pvName = new String("");
  protected JLabel     pvNameLabel;
  protected int        pvValue;
  protected JTextField pvValueTextField;
  protected JSlider    slider = null;
  protected int        sliderMax = 0;
  protected int        sliderMin = 0;
  protected int        oldEnumSelection = -1;
  protected JRadioButton menuButton[];

  /** Constructor with two arguments
  * It creates the Adjust Dialog
  * @param parent is the JFrame parent.
  * @param pv is the JprobePv.
  */
  JprobeAdjustDialog(JFrame parent,JprobePv pv)
  {
    super(parent);

    //Font monoFont = new Font("monospaced",Font.BOLD,12);
    //Font textFont = new Font("Courier",Font.BOLD,14);

    jprobePv = pv;
    pvName = jprobePv.getName();
    this.setTitle("Jprobe: Adjust Dialog");
    this.getContentPane().setLayout(new BorderLayout());

    /* Add a panel with a pvname label and text entry field */
    JPanel entryPanel = new JPanel();
    entryPanel.setLayout(new GridLayout(0,1));
    entryPanel.setBorder(new EmptyBorder(5,5,5,5));
    pvNameLabel = new JLabel(jprobePv.getName());
    //pvNameLabel.setFont(monoFont);
    entryPanel.add(pvNameLabel);
    JPanel valuePanel = new JPanel();
    pvValueTextField = new JTextField("",VALUE_TEXT_FIELD_LENGTH);
    pvValueTextField.setText(jprobePv.getValueString());
    //pvValueTextField.setFont(textFont);
    pvValueTextField.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String text = ((JTextField)e.getSource()).getText();
        if ( text != "") {
          try { 
	    jprobePv.setValue(text);
          }
          catch (jca.BadCountException bc){popup("jca.BadCountException"); }
          catch (jca.BadTypeException bt){popup("jca.BadTypeException"); }
          catch (jca.GetFailException bt){popup("jca.GetFailException"); }
          catch (jca.PutFailException bt){popup("jca.PutFailException"); }
          catch (jca.TimeOutException to){popup("jca.TimeOutException"); }
 
	}
	editing = false;
      }});
    pvValueTextField.addMouseListener(new MouseAdapter() {
      public void mouseEntered(MouseEvent e) { editing = true; }
      public void mouseExited(MouseEvent e) { editing = false; }
      });
    entryPanel.add(pvValueTextField);
    getContentPane().add(entryPanel, BorderLayout.NORTH);

    /* Add dbr type specific adjustPanel */
    adjustPanel = createAdjustPanel();
    if (adjustPanel != null ) {
      getContentPane().add(adjustPanel,BorderLayout.CENTER);
    }

    /* Add a panel with a close button */
    JPanel panel = new JPanel();
    JButton button = new JButton("Close");
    button.addActionListener( new ActionListener() {
      public void actionPerformed(ActionEvent e)
        { JprobeAdjustDialog.this.dispose(); } });
    Border raisedBorder = new BevelBorder(BevelBorder.RAISED);
    button.setBorder(new CompoundBorder(raisedBorder,new EmptyBorder(0,5,0,5)));
    panel.add(button);
    panel.setBorder(new EtchedBorder());
    this.getContentPane().add(panel, BorderLayout.SOUTH);

    updateDialog();

    pack();
    //this.setLocation(50,150);
    //this.setSize(220,180);
    setVisible(true);

    jprobePv.addValueEventListener(this);
  }

  public void valueChanged(ValueEvent e)
  {
//    synchronized(this) {
      jprobePv = e.jprobePv();
//    }
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
         updateDialog();
      }
    });
  }

  protected void updateDialog()
  {
    if (jprobePv.getName() != pvName){
      pvNameChanged();
    }
    if (!editing) {
      pvValueTextField.setText(jprobePv.getValueString());
    }
    switch (jprobePv.getFieldType()) {
      case DBR.SHORT:
      case DBR.FLOAT:
      case DBR.INT:
      case DBR.DOUBLE:
        if (slider == null) return;
        int newMax=((DBR_CTRL)jprobePv.getDBR()).upperCtrlLimit().intValue();
        int newMin=((DBR_CTRL)jprobePv.getDBR()).lowerCtrlLimit().intValue();
        if ( newMax != newMin ) {
          if ( newMax != sliderMax ) {
            sliderMax = newMax;
            slider.setMaximum(sliderMax);
          }
          if ( newMin < sliderMin ) {
            sliderMin = newMin;
            slider.setMinimum(sliderMin);
          }
	}
        int val = jprobePv.getDBR().dataAt(0).intValue();
        if ( val > sliderMax ) {
          sliderMax = val;
          slider.setMaximum(sliderMax);
        }
        if ( val < sliderMin ) {
          sliderMin = val;
          slider.setMinimum(sliderMin);
        }
        if ( sliderMax > sliderMin && val != pvValue ) {
             //slider.setValue(val);
	     pvValue = val;
        }
        break;
      case DBR.ENUM:
        DBR_INFO_DBEnum dbr = (DBR_INFO_DBEnum)jprobePv.getDBR();
        int i = dbr.dataAt(0).intValue();
        if ( i != oldEnumSelection) {
          oldEnumSelection = i;
          menuButton[i].setSelected(true);
        }
        break;
    }
  }

  protected void pvNameChanged()
  {
    pvName = jprobePv.getName();
    pvNameLabel.setText(pvName);
    editing = false;

    if (adjustPanel != null ) {
      getContentPane().remove(adjustPanel);
      adjustPanel = null;
    }
    adjustPanel = createAdjustPanel();
    if (adjustPanel != null ) {
      getContentPane().add(adjustPanel,BorderLayout.CENTER);
    }
    getContentPane().validate();
    pack();
  }

  protected JPanel createAdjustPanel()
  {
    switch (jprobePv.getFieldType()) {
      case DBR.SHORT:
      case DBR.FLOAT:
      case DBR.INT:
      case DBR.DOUBLE:
        pvValue = ((DBR)jprobePv.getDBR()).dataAt(0).intValue();
        sliderMin = ((DBR_CTRL)jprobePv.getDBR()).lowerCtrlLimit().intValue();
        sliderMax = ((DBR_CTRL)jprobePv.getDBR()).upperCtrlLimit().intValue();
        if (pvValue>sliderMax) sliderMax = pvValue;
        if (pvValue<sliderMin) sliderMin = pvValue;
        if (sliderMax <= sliderMin) return null;
        JPanel panel = new JPanel();
        Border valuatorBorder = new TitledBorder(null, "Valuator",
                TitledBorder.LEFT,TitledBorder.TOP);
        panel.setBorder(new CompoundBorder(valuatorBorder,
                                           new EmptyBorder(0,5,0,5)));
        panel.setLayout(new BorderLayout());
        slider = new JSlider(JSlider.HORIZONTAL,sliderMin,sliderMax,pvValue);
        slider.setMinorTickSpacing(2);
        slider.setMajorTickSpacing(10);
        slider.setPaintTicks(true);
        slider.setPaintLabels(true);
        slider.setLabelTable(slider.createStandardLabels(10));
        slider.addChangeListener(new ChangeListener() {
          public void stateChanged(ChangeEvent e) {
            if ( pvValue == slider.getValue() )  return;
            try { 
              jprobePv.setValue(slider.getValue());
            }
            catch (jca.BadCountException bc){popup("jca.BadCountException"); }
            catch (jca.BadTypeException bt){popup("jca.BadTypeException"); }
            catch (jca.GetFailException bt){popup("jca.GetFailException"); }
            catch (jca.PutFailException bm){popup("jca.PutFailException"); }
            catch (jca.TimeOutException to){popup("jca.TimeOutException"); }
          }});
        panel.add(slider,BorderLayout.CENTER);
        return panel;
      case DBR.STRING:
        return null;
      case DBR.BYTE:
        return null;
      case DBR.ENUM:
        DBR_INFO_DBEnum dbr = (DBR_INFO_DBEnum)jprobePv.getDBR();
        int count = dbr.nbStrings();
        String menuString[] = dbr.strings();
        menuButton = new JRadioButton[count + 1];
        JPanel menuPanel = new JPanel();
        menuPanel.setLayout(new GridLayout(0,4));
        Border menuBorder = new TitledBorder(null, "State",
                  TitledBorder.LEFT,TitledBorder.TOP);
        menuPanel.setBorder(new CompoundBorder(menuBorder,
                                               new EmptyBorder(0,5,0,5)));
        ButtonGroup group;
        group = new ButtonGroup();
        MenuRadioButtonListener menuListener =
          new MenuRadioButtonListener();
        for (int i=0;i<count;++i) {
          menuButton[i] = new JRadioButton(menuString[i],false);
          group.add(menuButton[i]);
          menuButton[i].addItemListener(menuListener);
          menuPanel.add(menuButton[i]);
        }
        return menuPanel;
    }
    return null;
  }

  protected void popup(String text)
  {
     JOptionPane.showMessageDialog(this,text,"OK",JOptionPane.ERROR_MESSAGE);
  }

  class MenuRadioButtonListener implements ItemListener
  {
    public void itemStateChanged(ItemEvent e)
    {
      String status;
      JRadioButton button = (JRadioButton) e.getItemSelectable();
      if(button.isSelected()) {
         try { 
           jprobePv.setValue(button.getText());
         }
         catch (jca.BadCountException bc){popup("jca.BadCountException"); }
         catch (jca.BadTypeException bt){popup("jca.BadTypeException"); }
         catch (jca.GetFailException bt){popup("jca.GetFailException"); }
         catch (jca.PutFailException bm){popup("jca.PutFailException"); }
         catch (jca.TimeOutException to){popup("jca.TimeOutException"); }
      }
    }
  }
}
