/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* JprobeFormatDialog.java,v 1.3 2002/08/02 15:39:23 jba Exp */
/**
 * This is the Format dialog.
 *
 * @author <A HREF="mailto:jba@aps.anl.gov">
 * Janet Anderson</A>
 *
 * @version 1.0
*/

package jprobe;

import javax.swing.*;
import javax.swing.border.*;

import java.awt.*;
import java.awt.event.*;

public class JprobeFormatDialog extends JDialog implements ValueEventListener
{
  protected static final int MAX_DECIMAL_PLACES = 15;

  protected JRadioButton precisionButton[] = new JRadioButton[MAX_DECIMAL_PLACES + 1];
  protected JprobePv     jprobePv;
  protected JRadioButton eNotationButton;
  protected JRadioButton fNotationButton;
  protected int          oldFormatPrecision = 0;
  protected String       oldFormatType = null;
  protected JRadioButton xNotationButton;

  JprobeFormatDialog(JFrame parent,JprobePv jPv)
  {
    super(parent);

    jprobePv = jPv;
    setTitle("Jprobe: Format Dialog");
    getContentPane().setLayout(new BorderLayout(10,10));
    Border emptyBorder = new EmptyBorder(0,5,0,5);
    JPanel panel = new JPanel();

    /* add a panel with 16 precision radio buttons */
    JPanel precisionPanel = new JPanel();
    precisionPanel.setLayout(new GridLayout(4,4));
    Border precisionBorder = new TitledBorder(null, "Decimal Places",
              TitledBorder.LEFT,TitledBorder.TOP);
    precisionPanel.setBorder(new CompoundBorder(precisionBorder,emptyBorder));
    ButtonGroup group;
    group = new ButtonGroup();
    PrecisionRadioButtonListener precisionListener =
                       new PrecisionRadioButtonListener();
    for (int i=0;i<=MAX_DECIMAL_PLACES;++i) {
      precisionButton[i] = new JRadioButton(Integer.toString(i),false);
      group.add(precisionButton[i]);
      precisionButton[i].addItemListener(precisionListener);
      precisionPanel.add(precisionButton[i]);
    }
    panel.add(precisionPanel);

    /* add a panel with notation radio buttons */
    JPanel formatPanel = new JPanel();
    formatPanel.setLayout(new GridLayout(0,1));
    Border formatBorder = new TitledBorder(null, "Notation",
              TitledBorder.LEFT,TitledBorder.TOP);
    formatPanel.setBorder(new CompoundBorder(formatBorder,emptyBorder));
    group = new ButtonGroup();
    NotationRadioButtonListener notationListener = 
      new NotationRadioButtonListener();
    fNotationButton = new JRadioButton("Fixed",false);
    fNotationButton.addItemListener(notationListener);
    group.add(fNotationButton);
    formatPanel.add(fNotationButton);
    eNotationButton = new JRadioButton("Scientific",false);
    eNotationButton.addItemListener(notationListener);
    group.add(eNotationButton);
    formatPanel.add(eNotationButton);
    xNotationButton = new JRadioButton("Hexadecimal",false);
    xNotationButton.addItemListener(notationListener);
    group.add(xNotationButton);
    //formatPanel.add(xNotationButton);
    //panel.add(formatPanel,BorderLayout.SOUTH);
    panel.add(formatPanel);
    getContentPane().add(panel,BorderLayout.CENTER);

    /* add a panel with a close button */
    JPanel buttonPanel = new JPanel();
    JButton closeButton = new JButton("Close");
    Border raisedBorder = new BevelBorder(BevelBorder.RAISED);
    closeButton.setBorder(new CompoundBorder(raisedBorder,emptyBorder));
    closeButton.addActionListener( new ActionListener() {
           public void actionPerformed(ActionEvent e) {
	     jprobePv.removeValueEventListener(JprobeFormatDialog.this);
	     JprobeFormatDialog.this.dispose();
	   } });
    buttonPanel.add(closeButton);
    buttonPanel.setBorder(new EtchedBorder());
    getContentPane().add(buttonPanel,BorderLayout.SOUTH);

    updateDialog();

    setLocation(100,100);
    //pack();
    setSize(200,320);
    setVisible(true);

    jprobePv.addValueEventListener(this);
  }

  public void valueChanged(ValueEvent e) {

      jprobePv = e.jprobePv();
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        updateDialog();
      }
    });
  }

  protected void updateDialog() {
    int i = jprobePv.getFormatPrecision();
    if ( i != oldFormatPrecision) {
      precisionButton[oldFormatPrecision].setSelected(false);
      oldFormatPrecision = i;
      precisionButton[i].setSelected(true);
    }
    String type = jprobePv.getFormatType();
    if ( !type.equals(oldFormatType)) {
      oldFormatType = type;
      if (type == "Scientific" ) {
        eNotationButton.setSelected(true);
      }
      else if (type == "Fixed" ) {
        fNotationButton.setSelected(true);
      }
      else if (type == "x" ) {
        xNotationButton.setSelected(true);
      }
    }
  }

  class PrecisionRadioButtonListener implements ItemListener {
    public void itemStateChanged(ItemEvent e){
      String status;
      JRadioButton button = (JRadioButton) e.getItemSelectable();
      if(button.isSelected()) {
         jprobePv.setFormatPrecision(
	     (new Integer(button.getText())).intValue());
      }
    }
  }

  class NotationRadioButtonListener implements ItemListener {
    public void itemStateChanged(ItemEvent e){
      String status;
      JRadioButton button = (JRadioButton) e.getItemSelectable();
      if(button.isSelected()) {
         jprobePv.setFormatType(button.getText());
      }
    }
  }
}
