/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* JprobePv.java,v 1.4 2002/08/02 15:39:23 jba Exp */
/**
 * This is the JprobePv class.
 *
 * @author <A HREF="mailto:jba@aps.anl.gov">
 * Janet Anderson</A>
 *
 * @version 1.0
*/

package jprobe;

import java.lang.String;
import java.util.*;
import java.io.*;
import java.text.DecimalFormat;
import javax.swing.event.*;

import jca.*;
import jca.event.*;
import jca.dbr.*;

public class JprobePv extends Object implements
  MonitorListener,ConnectionListener
{
  protected static final double CA_PEND_IO_TIME = 1.0;

  protected jca.dbr.DBR       dbr = null;
  protected DecimalFormat     fmt = new DecimalFormat();
  protected int               formatPrecision =3;
  protected String            formatType = "Fixed";
  protected EventListenerList listenerList = new EventListenerList();
  protected Monitor           monitor = null;
  protected Date              monitorStartTime = null;
  protected String            name = null;
  protected PV                pv = null;

  public JprobePv(String text,ValueEventListener listener) throws
    jca.BadCountException,
    jca.BadMonitorException,
    jca.BadTypeException,
    jca.GetFailException,
    jca.MonitorFailException,
    jca.PutFailException,
    jca.TimeOutException
  {
    this.addValueEventListener(listener);
    this.setName(text);
  }

  public synchronized void setName(String text) throws
    jca.BadCountException,
    jca.BadMonitorException,
    jca.BadTypeException,
    jca.GetFailException,
    jca.MonitorFailException,
    jca.PutFailException,
    jca.TimeOutException
  {

    if ( text.equals(name)) return;

    clear();
    name = text;

    if ( text.equals("")) {
      fireValueEvent();
      return;
    }
    pv= new PV(text);
    try { Ca.pendIO(CA_PEND_IO_TIME);}
    catch (jca.TimeOutException e) {}

    if (pv.state() == jca.PV.CONN ) {
      dbr = createDBR_STS(pv.fieldType());
      pv.get(dbr);
      Ca.pendIO(CA_PEND_IO_TIME);
      if ( dbr != null &&
		dbr.getClass() == DBR_CTRL_DBFloat.class) {
    	setFormatPrecision(((DBR_CTRL_DBFloat)dbr).precision());
      }
      if ( dbr != null &&
		dbr.getClass() == DBR_CTRL_DBDouble.class) {
    	setFormatPrecision(((DBR_CTRL_DBDouble)dbr).precision());
      }
    }
    fireValueEvent();
  }

  public synchronized void setMonitored(boolean bool) throws 
    jca.BadCountException,
    jca.BadMonitorException,
    jca.BadTypeException,
    jca.GetFailException,
    jca.MonitorFailException,
    jca.TimeOutException
  {
    if (pv == null || !isConnected()) {
    fireValueEvent();
    return;
    }

    if (bool && !isMonitored())
    {
      monitor = pv.addMonitor(dbr,this,null,Monitor.VALUE);
      Ca.pendIO(CA_PEND_IO_TIME);
      monitorStartTime = Calendar.getInstance().getTime();
    }
    else if (!bool && isMonitored() && monitor != null)
    {
      monitor.clear();
      Ca.pendIO(CA_PEND_IO_TIME);
      monitor = null;
      monitorStartTime = null;
    }
    fireValueEvent();
  }

  protected DBR_STS createDBR_STS(int type)
  {
    DBR_STS newDBR = null;
    switch (type) {
    case DBR.STRING:
      newDBR = new jca.dbr.DBR_STS_DBString();
      break;
    case DBR.SHORT:
      newDBR = new jca.dbr.DBR_CTRL_DBShort();
      break;
    case DBR.FLOAT:
      newDBR = new jca.dbr.DBR_CTRL_DBFloat();
      break;
    case DBR.ENUM:
      newDBR = new jca.dbr.DBR_INFO_DBEnum();
      break;
    case DBR.BYTE:
      newDBR = new jca.dbr.DBR_CTRL_DBByte();
      break;
    case DBR.INT:
      newDBR = new jca.dbr.DBR_CTRL_DBInt();
      break;
    case DBR.DOUBLE:
      newDBR = new jca.dbr.DBR_CTRL_DBDouble();
      break;
    }
    return newDBR;
  }

  public synchronized void setValue(int val) throws
    jca.BadCountException,
    jca.BadTypeException,
    jca.GetFailException,
    jca.PutFailException,
    jca.TimeOutException
  {
    setValue(new Integer(val).toString());
  }

  public synchronized void setValue(String text) throws 
    jca.BadCountException,
    jca.BadTypeException,
    jca.GetFailException,
    jca.PutFailException,
    jca.TimeOutException
  {
    if (pv == null || !isConnected()) return;
    pv.put(text);
    Ca.pendIO(CA_PEND_IO_TIME);
    if (!isMonitored() ) {
      pv.get(dbr);
      Ca.pendEvent(CA_PEND_IO_TIME);
      //Ca.pendIO(CA_PEND_IO_TIME);
      fireValueEvent();
    }
  }

  public synchronized boolean isMonitored()
  {
    if (monitor != null) return true;
    else return false;
  }
                                
  public synchronized boolean isConnected()
  {
    if ( pv != null && pv.state()==jca.PV.CONN ) return true;
    else return false;
  }

  public synchronized void setFormatPrecision(int i)
  {
    formatPrecision = i;
    if (formatType == null) return;
    if (formatType.equals("Fixed")) {
      fmt = new DecimalFormat();
      fmt.setMinimumFractionDigits(i);
      fmt.setMaximumFractionDigits(i);
    }
    else if (formatType.equals("Scientific")) {
      fmt = new DecimalFormat("##0.###E00");
      fmt.setMinimumFractionDigits(i);
      fmt.setMaximumFractionDigits(i);
      fmt.setMinimumIntegerDigits(1);
      fmt.setMaximumIntegerDigits(1);
    }
    fireValueEvent();
  }

  public synchronized void setFormatType(String text)
  {
    formatType = text;
    setFormatPrecision(formatPrecision);
  }

  protected synchronized void setDbr(jca.dbr.DBR newDbr) { dbr=newDbr; }

  public synchronized Date getMonitorStartTime() { return monitorStartTime; }
  public synchronized DecimalFormat getFormat() { return fmt; }
  public synchronized String   getFormatType() { return formatType; }
  public synchronized int      getFormatPrecision() { return formatPrecision; }
  public synchronized String   getName() { return name; }
  public synchronized jca.dbr.DBR  getDBR() { return dbr; }
  public synchronized PV  getPV() { return pv; }

  public synchronized int getFieldType()
  {
    if (pv == null) return -1;
    return pv.fieldType();
  }

  public synchronized int getState()
  {
    if (pv == null) return -1;
    return pv.state();
  }

  public synchronized String getAlarmStateString()
  {
    if (!isConnected()) return "";
    return Alarm.alarmSeverityString[((DBR_STS)dbr).severity()] + " " +
           Alarm.alarmStatusString[((DBR_STS)dbr).status()];
  }

  public synchronized String getUnitsString()
  {
     if (dbr == null) return "";
     Class dbrClass = dbr.getClass();
     if (dbrClass == DBR_CTRL_DBFloat.class || dbrClass == DBR_CTRL_DBDouble.class) {
       return ((DBR_CTRL)dbr).units();
     }
     return "";
  }

  public synchronized String getValueString()
  {
    if (pv==null || !isConnected()) return "Not connected";
    if (fmt != null && (pv.fieldType() == DBR.DOUBLE || pv.fieldType() == DBR.FLOAT)) {
      return fmt.format(dbr.dataAt(0).doubleValue());
    } else {
    	if (pv.fieldType() == DBR.ENUM ) {
          String menuString[] = ((DBR_INFO_DBEnum)dbr).strings();
          return menuString[dbr.dataAt(0).intValue()];
        }
      return dbr.dataAt(0).stringValue();
    }
  }

  public void connectionChanged(ConnectionEvent e)
  {
    //System.out.println("Connection event: name=" + e.pv().name() +
    //                   " state=" + e.op());
    fireValueEvent();
  }

  public synchronized void monitorChanged(MonitorEvent e)
  {
    //dbr = (DBR_STS)e.dbr();
    //dbr = e.dbr();
    setDbr(e.dbr());
    fireValueEvent();
  }

  public synchronized void addValueEventListener(ValueEventListener l) {
    listenerList.add(ValueEventListener.class, l);
    //fireValueEvent();
  }

  public synchronized void removeAllValueEventListeners() {
    Object[] listeners = listenerList.getListenerList();
    for (int i = listeners.length-2; i>=0; i-=2) {
      if (listeners[i]==ValueEventListener.class) {
    	listenerList.remove(ValueEventListener.class,
		(ValueEventListener)listeners[i+1]);
      }	       
    }	       
  }

  public synchronized void removeValueEventListener(ValueEventListener l) {
    listenerList.remove(ValueEventListener.class, l);
  }

  /*
   * Notify all listeners that have registered interest for
   * notification on this event type.  
   * @see EventListenerList
   */
  protected synchronized void fireValueEvent() {
    Object[] listeners = listenerList.getListenerList();
    for (int i = listeners.length-2; i>=0; i-=2) {
      if (listeners[i]==ValueEventListener.class) {
        ((ValueEventListener)listeners[i+1]).valueChanged(new ValueEvent(this,dbr));
      }	       
	}
  }

  public static void startCaLoop() {
    jca.util.PendEvent.start();
    //while (true) {
    //  Ca.pendEvent(1e-12);
    //}
  }

  public synchronized void clear() throws
    jca.BadMonitorException,
    jca.TimeOutException
  {
    if (isMonitored()) monitor.clear();
    monitor = null;
    monitorStartTime = null;
    if (pv != null) pv.clear();
    Ca.pendIO(CA_PEND_IO_TIME);
    pv = null;
    dbr = null;
  }

}
